@@ -1,10 +1,10 @@
 /* chown -- change user and group ownership of files
-   Copyright (C) 89, 90, 91, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 1995-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /*
               |		              user
@@ -37,7 +36,6 @@
 #include "chown-core.h"
 #include "error.h"
 #include "fts_.h"
-#include "lchown.h"
 #include "quote.h"
 #include "root-dev-ino.h"
 #include "userspec.h"
@@ -45,7 +43,9 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "chown"
 
-#define AUTHORS "David MacKenzie", "Jim Meyering"
+#define AUTHORS \
+  proper_name ("David MacKenzie"), \
+  proper_name ("Jim Meyering")
 
 /* The name the program was run with. */
 char *program_name;
@@ -156,7 +156,7 @@ Examples:\n\
   %s -hR root /u    Change the owner of /u and subfiles to \"root\".\n\
 "),
 	      program_name, program_name, program_name);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -240,7 +240,7 @@ main (int argc, char **argv)
 					     &required_uid, &required_gid,
 					     &u_dummy, &g_dummy);
 	    if (e)
-	      error (EXIT_FAILURE, 0, "%s: %s", quote (optarg), e);
+	      error (EXIT_FAILURE, 0, "%s: %s", e, quote (optarg));
 	    break;
 	  }
 
@@ -274,20 +274,14 @@ main (int argc, char **argv)
 	  if (dereference == 1)
 	    error (EXIT_FAILURE, 0,
 		   _("-R --dereference requires either -H or -L"));
-	  chopt.affect_symlink_referent = false;
-	}
-      else
-	{
-	  if (dereference == 0)
-	    error (EXIT_FAILURE, 0, _("-R -h requires -P"));
-	  chopt.affect_symlink_referent = true;
+	  dereference = 0;
 	}
     }
   else
     {
       bit_flags = FTS_PHYSICAL;
-      chopt.affect_symlink_referent = (dereference != 0);
     }
+  chopt.affect_symlink_referent = (dereference != 0);
 
   if (argc - optind < (reference_file ? 1 : 2))
     {
@@ -315,7 +309,7 @@ main (int argc, char **argv)
       const char *e = parse_user_spec (argv[optind], &uid, &gid,
 				       &chopt.user_name, &chopt.group_name);
       if (e)
-        error (EXIT_FAILURE, 0, "%s: %s", quote (argv[optind]), e);
+        error (EXIT_FAILURE, 0, "%s: %s", e, quote (argv[optind]));
 
       /* If a group is specified but no user, set the user name to the
 	 empty string so that diagnostics say "ownership :GROUP"
