@@ -1,10 +1,10 @@
 /* cksum -- calculate and print POSIX checksums and sizes of files
-   Copyright (C) 92, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 92, 1995-2006, 2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Q. Frank Xia, qx@math.columbia.edu.
    Cosmetic changes and reorganization by David MacKenzie, djm@gnu.ai.mit.edu.
@@ -38,16 +37,13 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "cksum"
 
-#define AUTHORS "Q. Frank Xia"
+#define AUTHORS proper_name ("Q. Frank Xia")
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <stdint.h>
 #include "system.h"
 
-#if !defined UINT_FAST32_MAX && !defined uint_fast32_t
-# define uint_fast32_t unsigned int
-#endif
-
 #ifdef CRCTAB
 
 # define BIT(x)	((uint_fast32_t) 1 << (x))
@@ -191,7 +187,7 @@ cksum (const char *file, bool print_name)
   uint_fast32_t crc = 0;
   uintmax_t length = 0;
   size_t bytes_read;
-  register FILE *fp;
+  FILE *fp;
   char length_buf[INT_BUFSIZE_BOUND (uintmax_t)];
   char const *hp;
 
@@ -199,10 +195,12 @@ cksum (const char *file, bool print_name)
     {
       fp = stdin;
       have_read_stdin = true;
+      if (O_BINARY && ! isatty (STDIN_FILENO))
+	freopen (NULL, "rb", stdin);
     }
   else
     {
-      fp = fopen (file, "r");
+      fp = fopen (file, (O_BINARY ? "rb" : "r"));
       if (fp == NULL)
 	{
 	  error (0, errno, "%s", file);
@@ -210,9 +208,6 @@ cksum (const char *file, bool print_name)
 	}
     }
 
-  /* Read input in BINARY mode, unless it is a console device.  */
-  SET_BINARY (fileno (fp));
-
   while ((bytes_read = fread (buf, 1, BUFLEN, fp)) > 0)
     {
       unsigned char *cp = buf;
@@ -277,7 +272,7 @@ Print CRC checksum and byte counts of each FILE.\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
