@@ -1,10 +1,10 @@
 /* comm -- compare two sorted files line by line.
-   Copyright (C) 86, 90, 91, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 86, 90, 91, 1995-2005, 2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,14 +12,12 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Richard Stallman and David MacKenzie. */
 
 #include <config.h>
 
-#include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
@@ -27,12 +25,15 @@
 #include "error.h"
 #include "hard-locale.h"
 #include "quote.h"
+#include "stdio--.h"
 #include "xmemcoll.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "comm"
 
-#define AUTHORS "Richard Stallman", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Richard Stallman"), \
+  proper_name ("David MacKenzie")
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
@@ -57,7 +58,7 @@ static struct option const long_options[] =
 {
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {0, 0, 0, 0}
+  {NULL, 0, NULL, 0}
 };
 
 
@@ -91,7 +92,7 @@ and column three contains lines common to both files.\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -137,10 +138,9 @@ writeline (const struct linebuffer *line, FILE *stream, int class)
 /* Compare INFILES[0] and INFILES[1].
    If either is "-", use the standard input for that file.
    Assume that each input file is sorted;
-   merge them and output the result.
-   Return true if successful.  */
+   merge them and output the result.  */
 
-static bool
+static void
 compare_files (char **infiles)
 {
   /* For each file, we have one linebuffer in lb1.  */
@@ -153,11 +153,7 @@ compare_files (char **infiles)
   /* streams[i] holds the input stream for file i.  */
   FILE *streams[2];
 
-  /* errno values for each stream.  */
-  int saved_errno[2];
-
   int i;
-  bool ret = true;
 
   /* Initialize the storage. */
   for (i = 0; i < 2; i++)
@@ -166,13 +162,11 @@ compare_files (char **infiles)
       thisline[i] = &lb1[i];
       streams[i] = (STREQ (infiles[i], "-") ? stdin : fopen (infiles[i], "r"));
       if (!streams[i])
-	{
-	  error (0, errno, "%s", infiles[i]);
-	  return false;
-	}
+	error (EXIT_FAILURE, errno, "%s", infiles[i]);
 
       thisline[i] = readlinebuffer (thisline[i], streams[i]);
-      saved_errno[i] = errno;
+      if (ferror (streams[i]))
+	error (EXIT_FAILURE, errno, "%s", infiles[i]);
     }
 
   while (thisline[0] || thisline[1])
@@ -214,31 +208,20 @@ compare_files (char **infiles)
       if (order >= 0)
 	{
 	  thisline[1] = readlinebuffer (thisline[1], streams[1]);
-	  saved_errno[1] = errno;
+	  if (ferror (streams[1]))
+	    error (EXIT_FAILURE, errno, "%s", infiles[1]);
 	}
       if (order <= 0)
 	{
 	  thisline[0] = readlinebuffer (thisline[0], streams[0]);
-	  saved_errno[0] = errno;
+	  if (ferror (streams[0]))
+	    error (EXIT_FAILURE, errno, "%s", infiles[0]);
 	}
     }
 
-  /* Free all storage and close all input streams. */
   for (i = 0; i < 2; i++)
-    {
-      free (lb1[i].buffer);
-      if (ferror (streams[i]))
-	{
-	  error (0, saved_errno[i], "%s", infiles[i]);
-	  ret = false;
-	}
-      if (fclose (streams[i]) != 0)
-	{
-	  error (0, errno, "%s", infiles[i]);
-	  ret = false;
-	}
-    }
-  return ret;
+    if (fclose (streams[i]) != 0)
+      error (EXIT_FAILURE, errno, "%s", infiles[i]);
 }
 
 int
@@ -297,6 +280,7 @@ main (int argc, char **argv)
       usage (EXIT_FAILURE);
     }
 
-  exit (compare_files (argv + optind)
-	? EXIT_SUCCESS : EXIT_FAILURE);
+  compare_files (argv + optind);
+
+  exit (EXIT_SUCCESS);
 }
