@@ -1,5 +1,5 @@
 /* cp.c  -- file copying (main routines)
-   Copyright (C) 89, 90, 91, 1995-2007 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,7 +52,10 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "cp"
 
-#define AUTHORS "Torbjorn Granlund", "David MacKenzie", "Jim Meyering"
+#define AUTHORS \
+  proper_name_utf8 ("Torbjorn Granlund", "Torbj\303\266rn Granlund"), \
+  proper_name ("David MacKenzie"), \
+  proper_name ("Jim Meyering")
 
 /* Used by do_copy, make_dir_parents_private, and re_protect
    to keep a list of leading directories whose protections
@@ -403,21 +406,43 @@ make_dir_parents_private (char const *const_dir, size_t src_offset,
 	slash++;
       while ((slash = strchr (slash, '/')))
 	{
-	  /* Add this directory to the list of directories whose modes need
-	     fixing later. */
-	  struct dir_attr *new = xmalloc (sizeof *new);
-	  new->slash_offset = slash - dir;
-	  new->restore_mode = false;
-	  new->next = *attr_list;
-	  *attr_list = new;
+	  struct dir_attr *new IF_LINT (= NULL);
+	  bool missing_dir;
 
 	  *slash = '\0';
-	  if (stat (dir, &stats) != 0)
+	  missing_dir = (stat (dir, &stats) != 0);
+
+	  if (missing_dir | x->preserve_ownership | x->preserve_mode
+	      | x->preserve_timestamps)
+	    {
+	      /* Add this directory to the list of directories whose
+		 modes might need fixing later. */
+	      struct stat src_st;
+	      int src_errno = (stat (src, &src_st) != 0
+			       ? errno
+			       : S_ISDIR (src_st.st_mode)
+			       ? 0
+			       : ENOTDIR);
+	      if (src_errno)
+		{
+		  error (0, src_errno, _("failed to get attributes of %s"),
+			 quote (src));
+		  return false;
+		}
+
+	      new = xmalloc (sizeof *new);
+	      new->st = src_st;
+	      new->slash_offset = slash - dir;
+	      new->restore_mode = false;
+	      new->next = *attr_list;
+	      *attr_list = new;
+	    }
+
+	  if (missing_dir)
 	    {
 	      mode_t src_mode;
 	      mode_t omitted_permissions;
 	      mode_t mkdir_mode;
-	      int src_errno;
 
 	      /* This component does not exist.  We must set
 		 *new_dst and new->st.st_mode inside this loop because,
@@ -425,17 +450,6 @@ make_dir_parents_private (char const *const_dir, size_t src_offset,
 		 make_dir_parents_private creates only e_dir/../a if
 		 ./b already exists. */
 	      *new_dst = true;
-	      src_errno = (stat (src, &new->st) != 0
-			   ? errno
-			   : S_ISDIR (new->st.st_mode)
-			   ? 0
-			   : ENOTDIR);
-	      if (src_errno)
-		{
-		  error (0, src_errno, _("failed to get attributes of %s"),
-			 quote (src));
-		  return false;
-		}
 	      src_mode = new->st.st_mode;
 
 	      /* If the ownership or special mode bits might change,
@@ -683,6 +697,16 @@ do_copy (int n_files, char **file, const char *target_directory,
 				  attr_list, x);
 	    }
 
+	  if (parents_option)
+	    {
+	      while (attr_list)
+		{
+		  struct dir_attr *p = attr_list;
+		  attr_list = attr_list->next;
+		  free (p);
+		}
+	    }
+
 	  free (dst_name);
 	}
     }
