@@ -1,10 +1,10 @@
 /* csplit - split a file into sections determined by context lines
-   Copyright (C) 91, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 91, 1995-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,15 +12,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Stuart Kemp, cpsrk@groper.jcu.edu.au.
    Modified by David MacKenzie, djm@gnu.ai.mit.edu. */
 
 #include <config.h>
 
-#include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include <signal.h>
@@ -30,20 +28,30 @@
 #include <regex.h>
 
 #include "error.h"
+#include "fd-reopen.h"
 #include "inttostr.h"
-#include "safe-read.h"
 #include "quote.h"
+#include "safe-read.h"
+#include "stdio--.h"
 #include "xstrtol.h"
 
+/* Use SA_NOCLDSTOP as a proxy for whether the sigaction machinery is
+   present.  */
 #ifndef SA_NOCLDSTOP
+# define SA_NOCLDSTOP 0
 # define sigprocmask(How, Set, Oset) /* empty */
 # define sigset_t int
+# if ! HAVE_SIGINTERRUPT
+#  define siginterrupt(sig, flag) /* empty */
+# endif
 #endif
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "csplit"
 
-#define AUTHORS "Stuart Kemp", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Stuart Kemp"), \
+  proper_name ("David MacKenzie")
 
 /* Increment size of area for control records. */
 #define ALLOC_SIZE 20
@@ -54,14 +62,14 @@
 /* A compiled pattern arg. */
 struct control
 {
-  char *regexpr;		/* Non-compiled regular expression. */
-  struct re_pattern_buffer re_compiled;	/* Compiled regular expression. */
   intmax_t offset;		/* Offset from regexp to split at. */
   uintmax_t lines_required;	/* Number of lines required. */
   uintmax_t repeat;		/* Repeat count. */
   int argnum;			/* ARGV index. */
   bool repeat_forever;		/* True if `*' used as a repeat count. */
   bool ignore;			/* If true, produce no output (for regexp). */
+  bool regexpr;			/* True if regular expression was used. */
+  struct re_pattern_buffer re_compiled;	/* Compiled regular expression. */
 };
 
 /* Initial size of data area in buffers. */
@@ -116,16 +124,13 @@ struct buffer_record
 
 static void close_output_file (void);
 static void create_output_file (void);
-static void delete_all_files (void);
+static void delete_all_files (bool);
 static void save_line_to_file (const struct cstring *line);
 void usage (int status);
 
 /* The name this program was run with. */
 char *program_name;
 
-/* Input file descriptor. */
-static int input_desc = 0;
-
 /* Start of buffer list. */
 static struct buffer_record *head = NULL;
 
@@ -145,13 +150,13 @@ static uintmax_t current_line = 0;
 static bool have_read_eof = false;
 
 /* Name of output files. */
-static char * volatile filename_space = NULL;
+static char *volatile filename_space = NULL;
 
 /* Prefix part of output file names. */
-static char * volatile prefix = NULL;
+static char const *volatile prefix = NULL;
 
 /* Suffix part of output file names. */
-static char * volatile suffix = NULL;
+static char *volatile suffix = NULL;
 
 /* Number of digits to use in output file names. */
 static int volatile digits = 2;
@@ -215,7 +220,7 @@ cleanup (void)
   close_output_file ();
 
   sigprocmask (SIG_BLOCK, &caught_signals, &oldset);
-  delete_all_files ();
+  delete_all_files (false);
   sigprocmask (SIG_SETMASK, &oldset, NULL);
 }
 
@@ -237,27 +242,22 @@ xalloc_die (void)
 static void
 interrupt_handler (int sig)
 {
-#ifndef SA_NOCLDSTOP
-  signal (sig, SIG_IGN);
-#endif
+  if (! SA_NOCLDSTOP)
+    signal (sig, SIG_IGN);
 
-  delete_all_files ();
+  delete_all_files (true);
 
   signal (sig, SIG_DFL);
   raise (sig);
 }
 
 /* Keep track of NUM bytes of a partial line in buffer START.
-   These bytes will be retrieved later when another large buffer is read.
-   It is not necessary to create a new buffer for these bytes; instead,
-   we keep a pointer to the existing buffer.  This buffer *is* on the
-   free list, and when the next buffer is obtained from this list
-   (even if it is this one), these bytes will be placed at the
-   start of the new buffer. */
+   These bytes will be retrieved later when another large buffer is read.  */
 
 static void
 save_to_hold_area (char *start, size_t num)
 {
+  free (hold_area);
   hold_area = start;
   hold_count = num;
 }
@@ -273,7 +273,7 @@ read_input (char *dest, size_t max_n_bytes)
   if (max_n_bytes == 0)
     return 0;
 
-  bytes_read = safe_read (input_desc, dest, max_n_bytes);
+  bytes_read = safe_read (STDIN_FILENO, dest, max_n_bytes);
 
   if (bytes_read == 0)
     have_read_eof = true;
@@ -383,7 +383,7 @@ record_line_starts (struct buffer_record *b)
 	  lines++;
 	}
       else
-	save_to_hold_area (line_start, bytes_left);
+	save_to_hold_area (xmemdup (line_start, bytes_left), bytes_left);
     }
 
   b->num_lines = lines;
@@ -439,6 +439,7 @@ static void
 free_buffer (struct buffer_record *buf)
 {
   free (buf->buffer);
+  buf->buffer = NULL;
 }
 
 /* Append buffer BUF to the linked list of buffers that contain
@@ -492,7 +493,7 @@ load_buffer (void)
   if (bytes_wanted < hold_count)
     bytes_wanted = hold_count;
 
-  do
+  while (1)
     {
       b = get_new_buffer (bytes_wanted);
       bytes_avail = b->bytes_alloc; /* Size of buffer returned. */
@@ -501,8 +502,7 @@ load_buffer (void)
       /* First check the `holding' area for a partial line. */
       if (hold_count)
 	{
-	  if (p != hold_area)
-	    memcpy (p, hold_area, hold_count);
+	  memcpy (p, hold_area, hold_count);
 	  p += hold_count;
 	  b->bytes_used += hold_count;
 	  bytes_avail -= hold_count;
@@ -512,14 +512,23 @@ load_buffer (void)
       b->bytes_used += read_input (p, bytes_avail);
 
       lines_found = record_line_starts (b);
-      bytes_wanted = b->bytes_alloc * 2;
       if (!lines_found)
 	free_buffer (b);
+
+      if (lines_found || have_read_eof)
+	break;
+
+      if (xalloc_oversized (2, b->bytes_alloc))
+	xalloc_die ();
+      bytes_wanted = 2 * b->bytes_alloc;
+      free_buffer (b);
+      free (b);
     }
-  while (!lines_found && !have_read_eof);
 
   if (lines_found)
     save_buffer (b);
+  else
+    free (b);
 
   return lines_found != 0;
 }
@@ -632,19 +641,13 @@ no_more_lines (void)
   return find_line (current_line + 1) == NULL;
 }
 
-/* Set the name of the input file to NAME and open it. */
+/* Open NAME as standard input.  */
 
 static void
 set_input_file (const char *name)
 {
-  if (STREQ (name, "-"))
-    input_desc = 0;
-  else
-    {
-      input_desc = open (name, O_RDONLY);
-      if (input_desc < 0)
-	error (EXIT_FAILURE, errno, "%s", name);
-    }
+  if (! STREQ (name, "-") && fd_reopen (STDIN_FILENO, name, O_RDONLY, 0) < 0)
+    error (EXIT_FAILURE, errno, _("cannot open %s for reading"), quote (name));
 }
 
 /* Write all lines from the beginning of the buffer up to, but
@@ -704,8 +707,8 @@ handle_line_error (const struct control *p, uintmax_t repetition)
 {
   char buf[INT_BUFSIZE_BOUND (uintmax_t)];
 
-  fprintf (stderr, _("%s: `%s': line number out of range"),
-	   program_name, umaxtostr (p->lines_required, buf));
+  fprintf (stderr, _("%s: %s: line number out of range"),
+	   program_name, quote (umaxtostr (p->lines_required, buf)));
   if (repetition)
     fprintf (stderr, _(" on repetition %s\n"), umaxtostr (repetition, buf));
   else
@@ -750,8 +753,8 @@ static void regexp_error (struct control *, uintmax_t, bool) ATTRIBUTE_NORETURN;
 static void
 regexp_error (struct control *p, uintmax_t repetition, bool ignore)
 {
-  fprintf (stderr, _("%s: `%s': match not found"),
-	   program_name, global_argv[p->argnum]);
+  fprintf (stderr, _("%s: %s: match not found"),
+	   program_name, quote (global_argv[p->argnum]));
 
   if (repetition)
     {
@@ -780,7 +783,7 @@ process_regexp (struct control *p, uintmax_t repetition)
   size_t line_len;		/* To make "$" in regexps work. */
   uintmax_t break_line;		/* First line number of next file. */
   bool ignore = p->ignore;	/* If true, skip this section. */
-  int ret;
+  regoff_t ret;
 
   if (!ignore)
     create_output_file ();
@@ -857,7 +860,7 @@ process_regexp (struct control *p, uintmax_t repetition)
 	      error (0, 0, _("error in regular expression search"));
 	      cleanup_fatal ();
 	    }
-	  if (ret >= 0)
+	  if (ret != -1)
 	    break;
 	}
     }
@@ -903,16 +906,21 @@ split_file (void)
   close_output_file ();
 }
 
-/* Return the name of output file number NUM. */
+/* Return the name of output file number NUM.
+
+   This function is called from a signal handler, so it should invoke
+   only reentrant functions that are async-signal-safe.  POSIX does
+   not guarantee this for the functions called below, but we don't
+   know of any hosts where this implementation isn't safe.  */
 
 static char *
 make_filename (unsigned int num)
 {
   strcpy (filename_space, prefix);
   if (suffix)
-    sprintf (filename_space+strlen(prefix), suffix, num);
+    sprintf (filename_space + strlen (prefix), suffix, num);
   else
-    sprintf (filename_space+strlen(prefix), "%0*u", digits, num);
+    sprintf (filename_space + strlen (prefix), "%0*u", digits, num);
   return filename_space;
 }
 
@@ -947,7 +955,7 @@ create_output_file (void)
    must be called only from critical sections.  */
 
 static void
-delete_all_files (void)
+delete_all_files (bool in_signal_handler)
 {
   unsigned int i;
 
@@ -957,7 +965,7 @@ delete_all_files (void)
   for (i = 0; i < files_created; i++)
     {
       const char *name = make_filename (i);
-      if (unlink (name))
+      if (unlink (name) != 0 && !in_signal_handler)
 	error (0, errno, "%s", name);
     }
 
@@ -974,7 +982,7 @@ close_output_file (void)
     {
       if (ferror (output_stream))
 	{
-	  error (0, 0, _("write error for `%s'"), output_filename);
+	  error (0, 0, _("write error for %s"), quote (output_filename));
 	  output_stream = NULL;
 	  cleanup_fatal ();
 	}
@@ -1031,9 +1039,9 @@ new_control_record (void)
   struct control *p;
 
   if (control_used == control_allocated)
-    controls = x2nrealloc (controls, &control_allocated, sizeof *controls);
+    controls = X2NREALLOC (controls, &control_allocated);
   p = &controls[control_used++];
-  p->regexpr = NULL;
+  p->regexpr = false;
   p->repeat = 0;
   p->repeat_forever = false;
   p->lines_required = 0;
@@ -1092,11 +1100,11 @@ parse_repeat_count (int argnum, struct control *p, char *str)
    Unless IGNORE is true, mark these lines for output. */
 
 static struct control *
-extract_regexp (int argnum, bool ignore, char *str)
+extract_regexp (int argnum, bool ignore, char const *str)
 {
   size_t len;			/* Number of bytes in this regexp. */
   char delim = *str;
-  char *closing_delim;
+  char const *closing_delim;
   struct control *p;
   const char *err;
 
@@ -1110,13 +1118,14 @@ extract_regexp (int argnum, bool ignore, char *str)
   p->argnum = argnum;
   p->ignore = ignore;
 
-  p->regexpr = xmalloc (len + 1);
-  strncpy (p->regexpr, str + 1, len);
-  p->re_compiled.allocated = len * 2;
-  p->re_compiled.buffer = xmalloc (p->re_compiled.allocated);
-  p->re_compiled.fastmap = xmalloc (1 << CHAR_BIT);
-  p->re_compiled.translate = 0;
-  err = re_compile_pattern (p->regexpr, len, &p->re_compiled);
+  p->regexpr = true;
+  p->re_compiled.buffer = NULL;
+  p->re_compiled.allocated = 0;
+  p->re_compiled.fastmap = xmalloc (UCHAR_MAX + 1);
+  p->re_compiled.translate = NULL;
+  re_syntax_options =
+    RE_SYNTAX_POSIX_BASIC & ~RE_CONTEXT_INVALID_DUP & ~RE_NO_EMPTY_RANGES;
+  err = re_compile_pattern (str + 1, len, &p->re_compiled);
   if (err)
     {
       error (0, 0, _("%s: invalid regular expression: %s"), str, err);
@@ -1161,14 +1170,14 @@ parse_patterns (int argc, int start, char **argv)
 	    {
 	      char buf[INT_BUFSIZE_BOUND (uintmax_t)];
 	      error (EXIT_FAILURE, 0,
-	       _("line number `%s' is smaller than preceding line number, %s"),
-		     argv[i], umaxtostr (last_val, buf));
+	       _("line number %s is smaller than preceding line number, %s"),
+		     quote (argv[i]), umaxtostr (last_val, buf));
 	    }
 
 	  if (val == last_val)
 	    error (0, 0,
-	   _("warning: line number `%s' is the same as preceding line number"),
-		   argv[i]);
+	   _("warning: line number %s is the same as preceding line number"),
+		   quote (argv[i]));
 
 	  last_val = val;
 
@@ -1266,7 +1275,7 @@ get_format_conv_type (char **format_ptr)
       break;
 
     default:
-      if (ISPRINT (ch))
+      if (isprint (ch))
         error (EXIT_FAILURE, 0,
 	       _("invalid conversion specifier in suffix: %c"), ch);
       else
@@ -1394,10 +1403,29 @@ main (int argc, char **argv)
 
   {
     int i;
-    static int const sig[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM };
+    static int const sig[] =
+      {
+	/* The usual suspects.  */
+	SIGALRM, SIGHUP, SIGINT, SIGPIPE, SIGQUIT, SIGTERM,
+#ifdef SIGPOLL
+	SIGPOLL,
+#endif
+#ifdef SIGPROF
+	SIGPROF,
+#endif
+#ifdef SIGVTALRM
+	SIGVTALRM,
+#endif
+#ifdef SIGXCPU
+	SIGXCPU,
+#endif
+#ifdef SIGXFSZ
+	SIGXFSZ,
+#endif
+      };
     enum { nsigs = sizeof sig / sizeof sig[0] };
 
-#ifdef SA_NOCLDSTOP
+#if SA_NOCLDSTOP
     struct sigaction act;
 
     sigemptyset (&caught_signals);
@@ -1418,13 +1446,16 @@ main (int argc, char **argv)
 #else
     for (i = 0; i < nsigs; i++)
       if (signal (sig[i], SIG_IGN) != SIG_IGN)
-	signal (sig[i], interrupt_handler);
+	{
+	  signal (sig[i], interrupt_handler);
+	  siginterrupt (sig[i], 1);
+	}
 #endif
   }
 
   split_file ();
 
-  if (close (input_desc) < 0)
+  if (close (STDIN_FILENO) != 0)
     {
       error (0, errno, _("read error"));
       cleanup_fatal ();
@@ -1454,7 +1485,7 @@ and output byte counts of each piece to standard output.\n\
 Mandatory arguments to long options are mandatory for short options too.\n\
 "), stdout);
       fputs (_("\
-  -b, --suffix-format=FORMAT use sprintf FORMAT instead of %02d\n\
+  -b, --suffix-format=FORMAT  use sprintf FORMAT instead of %02d\n\
   -f, --prefix=PREFIX        use PREFIX instead of `xx'\n\
   -k, --keep-files           do not remove output files on errors\n\
 "), stdout);
@@ -1479,7 +1510,7 @@ Read standard input if FILE is -.  Each PATTERN may be:\n\
 \n\
 A line OFFSET is a required `+' or `-' followed by a positive integer.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
