@@ -1,10 +1,10 @@
 /* tee - read from standard input and write to standard output and files.
-   Copyright (C) 85,1990-2006 Free Software Foundation, Inc.
+   Copyright (C) 85,1990-2006, 2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Mike Parker, Richard M. Stallman, and David MacKenzie */
 
@@ -29,7 +28,10 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "tee"
 
-#define AUTHORS "Mike Parker", "Richard M. Stallman", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Mike Parker"), \
+  proper_name ("Richard M. Stallman"), \
+  proper_name ("David MacKenzie")
 
 static bool tee_files (int nfiles, const char **files);
 
@@ -72,7 +74,7 @@ Copy standard input to each FILE, and also to standard output.\n\
 \n\
 If a FILE is -, copy again to standard output.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -161,7 +163,7 @@ tee_files (int nfiles, const char **files)
      the first one correspond to standard output.   */
   descriptors[0] = stdout;
   files[0] = _("standard output");
-  SETVBUF (stdout, NULL, _IONBF, 0);
+  setvbuf (stdout, NULL, _IONBF, 0);
 
   for (i = 1; i <= nfiles; i++)
     {
@@ -174,7 +176,7 @@ tee_files (int nfiles, const char **files)
 	  ok = false;
 	}
       else
-	SETVBUF (descriptors[i], NULL, _IONBF, 0);
+	setvbuf (descriptors[i], NULL, _IONBF, 0);
     }
 
   while (1)
