@@ -1,10 +1,10 @@
 /* uniq -- remove duplicate lines from a sorted file
-   Copyright (C) 86, 91, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 86, 91, 1995-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Richard Stallman and David MacKenzie. */
 
@@ -23,16 +22,6 @@
 #include <getopt.h>
 #include <sys/types.h>
 
-/* Get mbstate_t, mbrtowc(). */
-#if HAVE_WCHAR_H
-# include <wchar.h>
-#endif
-
-/* Get isw* functions. */
-#if HAVE_WCTYPE_H
-# include <wctype.h>
-#endif
-
 #include "system.h"
 #include "argmatch.h"
 #include "linebuffer.h"
@@ -42,24 +31,14 @@
 #include "quote.h"
 #include "xmemcoll.h"
 #include "xstrtol.h"
-#include "xmemcoll.h"
-
-/* MB_LEN_MAX is incorrectly defined to be 1 in at least one GCC
-   installation; work around this configuration error.  */
-#if !defined MB_LEN_MAX || MB_LEN_MAX < 2
-# define MB_LEN_MAX 16
-#endif
-
-/* Some systems, like BeOS, have multibyte encodings but lack mbstate_t.  */
-#if HAVE_MBRTOWC && defined mbstate_t
-# define mbrtowc(pwc, s, n, ps) (mbrtowc) (pwc, s, n, 0)
-#endif
-
+#include "memcasecmp.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "uniq"
 
-#define AUTHORS "Richard Stallman", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Richard Stallman"), \
+  proper_name ("David MacKenzie")
 
 #define SWAP_LINES(A, B)			\
   do						\
@@ -131,10 +110,6 @@ static enum delimit_method const delimit_method_map[] =
 /* Select whether/how to delimit groups of duplicate lines.  */
 static enum delimit_method delimit_groups;
 
-/* Function pointers. */
-static char *
-(*find_field) (struct linebuffer *line);
-
 static struct option const longopts[] =
 {
   {"count", no_argument, NULL, 'c'},
@@ -145,6 +120,7 @@ static struct option const longopts[] =
   {"skip-fields", required_argument, NULL, 'f'},
   {"skip-chars", required_argument, NULL, 's'},
   {"check-chars", required_argument, NULL, 'w'},
+  {"zero-terminated", no_argument, NULL, 'z'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
@@ -182,6 +158,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -i, --ignore-case     ignore differences in case when comparing\n\
   -s, --skip-chars=N    avoid comparing the first N characters\n\
   -u, --unique          only print unique lines\n\
+  -z, --zero-terminated  end lines with 0 byte, not newline\n\
 "), stdout);
      fputs (_("\
   -w, --check-chars=N   compare no more than N characters in lines\n\
@@ -190,10 +167,15 @@ Mandatory arguments to long options are mandatory for short options too.\n\
      fputs (VERSION_OPTION_DESCRIPTION, stdout);
      fputs (_("\
 \n\
-A field is a run of whitespace, then non-whitespace characters.\n\
-Fields are skipped before chars.\n\
+A field is a run of blanks (usually spaces and/or TABs), then non-blank\n\
+characters.  Fields are skipped before chars.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+     fputs (_("\
+\n\
+Note: 'uniq' does not detect repeated lines unless they are adjacent.\n\
+You may want to sort the input first, or use `sort -u' without `uniq'.\n\
+"), stdout);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -224,7 +206,7 @@ size_opt (char const *opt, char const *msgid)
    return a pointer to the beginning of the line's field to be compared. */
 
 static char *
-find_field_uni (struct linebuffer *line)
+find_field (const struct linebuffer *line)
 {
   size_t count;
   char *lp = line->buffer;
@@ -233,9 +215,9 @@ find_field_uni (struct linebuffer *line)
 
   for (count = 0; count < skip_fields && i < size; count++)
     {
-      while (i < size && isblank (lp[i]))
+      while (i < size && isblank (to_uchar (lp[i])))
 	i++;
-      while (i < size && !isblank (lp[i]))
+      while (i < size && !isblank (to_uchar (lp[i])))
 	i++;
     }
 
@@ -245,83 +227,6 @@ find_field_uni (struct linebuffer *line)
   return lp + i;
 }
 
-#if HAVE_MBRTOWC
-
-# define MBCHAR_TO_WCHAR(WC, MBLENGTH, LP, POS, SIZE, STATEP, CONVFAIL)  \
-  do									\
-    {									\
-      mbstate_t state_bak;						\
-									\
-      CONVFAIL = 0;							\
-      state_bak = *STATEP;						\
-									\
-      MBLENGTH = mbrtowc (&WC, LP + POS, SIZE - POS, STATEP);		\
-									\
-      switch (MBLENGTH)							\
-	{								\
-	case (size_t)-2:						\
-	case (size_t)-1:						\
-	  *STATEP = state_bak;						\
-	  CONVFAIL++;							\
-	  /* Fall through */						\
-	case 0:								\
-	  MBLENGTH = 1;							\
-	}								\
-    }									\
-  while (0)
-
-static char *
-find_field_multi (struct linebuffer *line)
-{
-  size_t count;
-  char *lp = line->buffer;
-  size_t size = line->length - 1;
-  size_t pos;
-  size_t mblength;
-  wchar_t wc;
-  mbstate_t *statep;
-  int convfail;
-
-  pos = 0;
-  statep = &(line->state);
-
-  /* skip fields. */
-  for (count = 0; count < skip_fields && pos < size; count++)
-    {
-      while (pos < size)
-	{
-	  MBCHAR_TO_WCHAR (wc, mblength, lp, pos, size, statep, convfail);
- 
-	  if (convfail || !iswblank (wc))
-	    {
-	      pos += mblength;
-	      break;
-	    }
-	  pos += mblength;
-	}
-
-      while (pos < size)
-	{
-	  MBCHAR_TO_WCHAR (wc, mblength, lp, pos, size, statep, convfail);
-
-	  if (!convfail && iswblank (wc))
-	    break;
-
-	  pos += mblength;
-	}
-    }
-
-  /* skip fields. */
-  for (count = 0; count < skip_chars && pos < size; count++)
-    {
-      MBCHAR_TO_WCHAR (wc, mblength, lp, pos, size, statep, convfail);
-      pos += mblength;
-    }
-
-  return lp + pos;
-}
-#endif
-
 /* Return false if two strings OLD and NEW match, true if not.
    OLD and NEW point not to the beginnings of the lines
    but rather to the beginnings of the fields to compare.
@@ -330,8 +235,6 @@ find_field_multi (struct linebuffer *line)
 static bool
 different (char *old, char *new, size_t oldlen, size_t newlen)
 {
-  char *copy_old, *copy_new;
-
   if (check_chars < oldlen)
     oldlen = check_chars;
   if (check_chars < newlen)
@@ -339,93 +242,15 @@ different (char *old, char *new, size_t oldlen, size_t newlen)
 
   if (ignore_case)
     {
-      size_t i;
-
-      copy_old = alloca (oldlen + 1);
-      copy_new = alloca (oldlen + 1);
-
-      for (i = 0; i < oldlen; i++)
-	{
-	  copy_old[i] = toupper (old[i]);
-	  copy_new[i] = toupper (new[i]);
-	}
+      /* FIXME: This should invoke strcoll somehow.  */
+      return oldlen != newlen || memcasecmp (old, new, oldlen);
     }
+  else if (hard_LC_COLLATE)
+    return xmemcoll (old, oldlen, new, newlen) != 0;
   else
-    {
-      copy_old = (char *)old;
-      copy_new = (char *)new;
-    }
-
-  return xmemcoll (copy_old, oldlen, copy_new, newlen);
+    return oldlen != newlen || memcmp (old, new, oldlen);
 }
 
-#if HAVE_MBRTOWC
-static int
-different_multi (const char *old, const char *new, size_t oldlen, size_t newlen, mbstate_t oldstate, mbstate_t newstate)
-{
-  size_t i, j, chars;
-  const char *str[2];
-  char *copy[2];
-  size_t len[2];
-  mbstate_t state[2];
-  size_t mblength;
-  wchar_t wc, uwc;
-  mbstate_t state_bak;
-
-  str[0] = old;
-  str[1] = new;
-  len[0] = oldlen;
-  len[1] = newlen;
-  state[0] = oldstate;
-  state[1] = newstate;
-
-  for (i = 0; i < 2; i++)
-    {
-      copy[i] = alloca (len[i] + 1);
-
-      for (j = 0, chars = 0; j < len[i] && chars < check_chars; chars++)
-	{
-	  state_bak = state[i];
-	  mblength = mbrtowc (&wc, str[i] + j, len[i] - j, &(state[i]));
-
-	  switch (mblength)
-	    {
-	    case (size_t)-1:
-	    case (size_t)-2:
-	      state[i] = state_bak;
-	      /* Fall through */
-	    case 0:
-	      mblength = 1;
-	      break;
-
-	    default:
-	      if (ignore_case)
-		{
-		  uwc = towupper (wc);
-
-		  if (uwc != wc)
-		    {
-		      mbstate_t state_wc;
-
-		      memset (&state_wc, '\0', sizeof(mbstate_t));
-		      wcrtomb (copy[i] + j, uwc, &state_wc);
-		    }
-		  else
-		    memcpy (copy[i] + j, str[i] + j, mblength);
-		}
-	      else
-		memcpy (copy[i] + j, str[i] + j, mblength);
-	    }
-	  j += mblength;
-	}
-      copy[i][j] = '\0';
-      len[i] = j;
-    }
-
-  return xmemcoll (copy[0], len[0], copy[1], len[1]);
-}
-#endif
-
 /* Output the line in linebuffer LINE to standard output
    provided that the switches say it should be output.
    MATCH is true if the line matches the previous line.
@@ -451,7 +276,7 @@ writeline (struct linebuffer const *line,
    If either is "-", use the standard I/O stream for it instead. */
 
 static void
-check_file (const char *infile, const char *outfile)
+check_file (const char *infile, const char *outfile, char delimiter)
 {
   struct linebuffer lb1, lb2;
   struct linebuffer *thisline, *prevline;
@@ -478,43 +303,15 @@ check_file (const char *infile, const char *outfile)
     {
       char *prevfield IF_LINT (= NULL);
       size_t prevlen IF_LINT (= 0);
-#if HAVE_MBRTOWC
-      mbstate_t prevstate;
-
-      memset (&prevstate, '\0', sizeof (mbstate_t));
-#endif
 
       while (!feof (stdin))
 	{
 	  char *thisfield;
 	  size_t thislen;
-#if HAVE_MBRTOWC
-	  mbstate_t thisstate;
-#endif
-
-	  if (readlinebuffer (thisline, stdin) == 0)
+	  if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
 	    break;
 	  thisfield = find_field (thisline);
 	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
-#if HAVE_MBRTOWC
-	  if (MB_CUR_MAX > 1)
-            {
-            thisstate = thisline->state;
-
-            if (prevline->length == 0 || different_multi
-              (thisfield, prevfield, thislen, prevlen, thisstate, prevstate))
-              {
-                fwrite (thisline->buffer, sizeof (char),
-                        thisline->length, stdout);
-
-                SWAP_LINES (prevline, thisline);
-                prevfield = thisfield;
-                prevlen = thislen;
-                prevstate = thisstate;
-              }
-          }
-	else
-#endif
 	  if (prevline->length == 0
 	      || different (thisfield, prevfield, thislen, prevlen))
 	    {
@@ -533,27 +330,18 @@ check_file (const char *infile, const char *outfile)
       size_t prevlen;
       uintmax_t match_count = 0;
       bool first_delimiter = true;
-#if HAVE_MBRTOWC
-      mbstate_t prevstate;
-#endif
 
-      if (readlinebuffer (prevline, stdin) == 0)
+      if (readlinebuffer_delim (prevline, stdin, delimiter) == 0)
 	goto closefiles;
       prevfield = find_field (prevline);
       prevlen = prevline->length - 1 - (prevfield - prevline->buffer);
-#if HAVE_MBRTOWC
-      prevstate = prevline->state;
-#endif
 
       while (!feof (stdin))
 	{
 	  bool match;
 	  char *thisfield;
 	  size_t thislen;
-#if HAVE_MBRTOWC
-	  mbstate_t thisstate;
-#endif
-	  if (readlinebuffer (thisline, stdin) == 0)
+	  if (readlinebuffer_delim (thisline, stdin, delimiter) == 0)
 	    {
 	      if (ferror (stdin))
 		goto closefiles;
@@ -561,15 +349,6 @@ check_file (const char *infile, const char *outfile)
 	    }
 	  thisfield = find_field (thisline);
 	  thislen = thisline->length - 1 - (thisfield - thisline->buffer);
-#if HAVE_MBRTOWC
-	  if (MB_CUR_MAX > 1)
-	    {
-              thisstate = thisline->state;
-              match = !different_multi (thisfield, prevfield,
-                                thislen, prevlen, thisstate, prevstate);
-            }
-	  else
-#endif
 	  match = !different (thisfield, prevfield, thislen, prevlen);
 	  match_count += match;
 
@@ -592,7 +371,7 @@ check_file (const char *infile, const char *outfile)
 		  if ((delimit_groups == DM_PREPEND)
 		      || (delimit_groups == DM_SEPARATE
 			  && !first_delimiter))
-		    putchar ('\n');
+		    putchar (delimiter);
 		}
 	    }
 
@@ -602,9 +381,6 @@ check_file (const char *infile, const char *outfile)
 	      SWAP_LINES (prevline, thisline);
 	      prevfield = thisfield;
 	      prevlen = thislen;
-#if HAVE_MBRTOWC
-	      prevstate = thisstate;
-#endif
 	      if (!match)
 		match_count = 0;
 	    }
@@ -638,6 +414,7 @@ main (int argc, char **argv)
   enum Skip_field_option_type skip_field_option_type = SFO_NONE;
   int nfiles = 0;
   char const *file[2];
+  char delimiter = '\n';	/* change with --zero-terminated, -z */
 
   file[0] = file[1] = "-";
   initialize_main (&argc, &argv);
@@ -649,19 +426,6 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-#if HAVE_MBRTOWC
-  if (MB_CUR_MAX > 1)
-    {
-      find_field = find_field_multi;
-    }
-  else
-#endif
-    {
-      find_field = find_field_uni;
-    }
-
-
-
   skip_chars = 0;
   skip_fields = 0;
   check_chars = SIZE_MAX;
@@ -679,7 +443,7 @@ main (int argc, char **argv)
       if (optc == -1
 	  || (posixly_correct && nfiles != 0)
 	  || ((optc = getopt_long (argc, argv,
-				   "-0123456789Dcdf:is:uw:", longopts, NULL))
+				   "-0123456789Dcdf:is:uw:z", longopts, NULL))
 	      == -1))
 	{
 	  if (argc <= optind)
@@ -775,6 +539,10 @@ main (int argc, char **argv)
 				  N_("invalid number of bytes to compare"));
 	  break;
 
+	case 'z':
+	  delimiter = '\0';
+	  break;
+
 	case_GETOPT_HELP_CHAR;
 
 	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
@@ -791,7 +559,7 @@ main (int argc, char **argv)
       usage (EXIT_FAILURE);
     }
 
-  check_file (file[0], file[1]);
+  check_file (file[0], file[1], delimiter);
 
   exit (EXIT_SUCCESS);
 }
