@@ -1,5 +1,5 @@
 /* GNU's who.
-   Copyright (C) 1992-2007 Free Software Foundation, Inc.
+   Copyright (C) 1992-2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,7 +40,10 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "who"
 
-#define AUTHORS "Joseph Arceneaux", "David MacKenzie", "Michael Stone"
+#define AUTHORS \
+  proper_name ("Joseph Arceneaux"), \
+  proper_name ("David MacKenzie"), \
+  proper_name ("Michael Stone")
 
 #ifndef MAXHOSTNAMELEN
 # define MAXHOSTNAMELEN 64
